
#
# Copyright (c) 2011, MegaVersiya, LTD. All rights reserved.
#
# Redistribution and use in source and binary forms, with or
# without modification, are permitted provided that
# the following conditions are met:
# Redistributions of source code must retain the above copyright notice,
# this list of conditions and the following disclaimer.
# Redistributions in binary form must reproduce the above copyright notice,
# this list of conditions and the following disclaimer in the documentation
# and/or other materials provided with the distribution.
# Neither the name of the MegaVersiya, LTD nor the names of its contributors
# may be used to endorse or promote products derived from this software
# without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
# OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
# OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT * LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
# OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
# DAMAGE.
#

import httplib

from django.test import TestCase
from django.core.urlresolvers import reverse
from django.conf import settings
from django.utils.translation import ugettext_lazy as _

from utils.test import BaseViewTestCase
from admins.models import Admin
from admins.forms import AddAdminForm, AdminPasswordChangeForm


class NoAuthAdminViewTestCase(TestCase):
    def test_admin_list(self):
        response = self.client.get(reverse('admins.views.admin_list'))

        self.assertRedirects(response, settings.LOGIN_URL)

    def test_add_admin(self):
        response = self.client.get(reverse('admins.views.add_admin'))

        self.assertRedirects(response, settings.LOGIN_URL)

    def test_change_password(self):
        response = self.client.get(
            reverse('admins.views.change_password', args=(1,))
        )

        self.assertRedirects(response, settings.LOGIN_URL)

    def test_delete_admin(self):
        response = self.client.get(
            reverse('admins.views.delete_admin', args=(1,))
        )

        self.assertRedirects(response, settings.LOGIN_URL)


class AdminListViewTestCase(BaseViewTestCase):
    def test_admin_list_for_general(self):
        self.switch_current_admin_role(settings.ROLE_GENERAL)

        response = self.client.get(reverse('admins.views.admin_list'))

        self.assertEqual(response.status_code, httplib.OK)
        self.assertTemplateUsed(response, 'admins/admin_list.html')

        # Admin list now contents one item - current admin.
        self.assertEqual(len(response.context['admin_list']), 1)

        self.assertEqual(
            response.context['current_admin'], self.get_current_admin()
        )

        self.assertTrue(response.context['can_add'])
        self.assertTrue(response.context['can_delete'])

    def test_admin_list_for_custom(self):
        self.switch_current_admin_role(settings.ROLE_CUSTOM)

        response = self.client.get(reverse('admins.views.admin_list'))

        self.assertEqual(response.status_code, httplib.OK)
        self.assertTemplateUsed(response, 'admins/admin_list.html')

        # Admin list now contents one item - current admin.
        self.assertTrue(len(response.context['admin_list']), 1)

        self.assertEqual(
            response.context['current_admin'], self.get_current_admin()
        )

        self.assertFalse(response.context['can_add'])
        self.assertFalse(response.context['can_delete'])

    def test_admin_list_many_items_for_general(self):
        self.switch_current_admin_role(settings.ROLE_GENERAL)
        admin_count = 3
        for i in range(admin_count):
            admin = Admin()
            admin.aname = 'test_name_%d' % i
            admin.save()

        response = self.client.get(reverse('admins.views.admin_list'))

        # Admin list now contents four items - current admin + 3 new.
        self.assertEqual(
            len(response.context['admin_list']),
            admin_count + 1
        )

    def test_admin_list_many_items_for_custom(self):
        self.switch_current_admin_role(settings.ROLE_CUSTOM)
        admin_count = 3
        for i in range(admin_count):
            admin = Admin()
            admin.aname = 'test_name_%d' % i
            admin.save()

        response = self.client.get(reverse('admins.views.admin_list'))

        # Custom admin list now contents only current admin item.
        self.assertEqual(
            len(response.context['admin_list']), 1
        )


class AddAdminViewTestCase(BaseViewTestCase):
    def test_add_admin_for_custom(self):
        self.switch_current_admin_role(settings.ROLE_CUSTOM)

        response = self.client.get(reverse('admins.views.add_admin'))

        self.assertEqual(response.status_code, httplib.FORBIDDEN)

    def test_add_admin_for_general(self):
        self.switch_current_admin_role(settings.ROLE_GENERAL)

        response = self.client.get(reverse('admins.views.add_admin'))

        self.assertEqual(response.status_code, httplib.OK)
        self.assertTemplateUsed(response, 'admins/add_admin.html')

        self.assertEqual(
            type(response.context['form']), AddAdminForm
        )

    def test_add_admin_send_empty_data(self):
        self.switch_current_admin_role(settings.ROLE_GENERAL)

        data = {
            'aname': '',
            'password1': '',
            'password2': ''
        }

        response = self.client.post(
            reverse('admins.views.add_admin'), data
        )

        self.assertFormError(
            response, 'form', 'aname',
            _(u'This field is required.')
        )
        self.assertFormError(
            response,
            'form',
            'password1',
            _(u'This field is required.')
        )
        self.assertFormError(
            response,
            'form',
            'password2',
            _(u'This field is required.')
        )

    def test_add_admin_send_password_confirm_error(self):
        self.switch_current_admin_role(settings.ROLE_GENERAL)

        data = {
            'aname': 'test_name',
            'password1': 'password1',
            'password2': 'password2'
        }

        response = self.client.post(
            reverse('admins.views.add_admin'), data
        )

        self.assertFormError(
            response, 'form', 'password2',
            _(u"The two password fields didn't match.")
        )

    def test_add_admin_send_success(self):
        self.switch_current_admin_role(settings.ROLE_GENERAL)

        data = {
            'aname': 'test_name',
            'password1': 'password',
            'password2': 'password'
        }

        response = self.client.post(
            reverse('admins.views.add_admin'), data
        )

        self.assertRedirects(response, reverse('admins.views.admin_list'))

        admin = Admin.objects.get(aname=data['aname'])

        self.assertEqual(admin.passwd, data['password1'])

        self.assertEqual(admin.type, settings.ROLE_CUSTOM)


class ChangePasswordViewTestCase(BaseViewTestCase):
    def test_change_password_not_found(self):
        self.switch_current_admin_role(settings.ROLE_GENERAL)

        not_exist_admin_pk = 9999

        response = self.client.get(
            reverse(
                'admins.views.change_password',
                args=(not_exist_admin_pk,)
            )
        )

        self.assertEqual(response.status_code, httplib.NOT_FOUND)

    def test_change_password_forbidden(self):
        self.switch_current_admin_role(settings.ROLE_CUSTOM)

        not_exist_admin_pk = 9999

        response = self.client.get(
            reverse(
                'admins.views.change_password',
                args=(not_exist_admin_pk,)
            )
        )

        self.assertEqual(response.status_code, httplib.FORBIDDEN)

    def test_change_password(self):
        response = self.client.get(
            reverse(
                'admins.views.change_password',
                args=(self.admin_pk,)
            )
        )

        self.assertEqual(response.status_code, httplib.OK)
        self.assertTemplateUsed(response, 'admins/change_password.html')

        self.assertEqual(
            type(response.context['form']), AdminPasswordChangeForm
        )

    def test_change_password_send_empty_data(self):
        self.switch_current_admin_role(settings.ROLE_GENERAL)

        admin = Admin()
        admin.aname = 'test_name'
        admin.save()

        data = {
            'password1': '',
            'password2': ''
        }

        response = self.client.post(
            reverse(
                'admins.views.change_password',
                args=(admin.pk,)
            ), data
        )

        self.assertFormError(
            response,
            'form',
            'password1',
            _(u'This field is required.')
        )
        self.assertFormError(
            response,
            'form',
            'password2',
            _(u'This field is required.')
        )

    def test_change_password_send_password_confirm_error(self):
        self.switch_current_admin_role(settings.ROLE_GENERAL)

        admin = Admin()
        admin.aname = 'test_name'
        admin.save()

        data = {
            'password1': 'password1',
            'password2': 'password2'
        }

        response = self.client.post(
            reverse(
                'admins.views.change_password',
                args=(admin.pk,)
            ), data
        )

        self.assertFormError(
            response, 'form', 'password2',
            _(u"The two password fields didn't match.")
        )

    def test_change_password_send_success(self):
        self.switch_current_admin_role(settings.ROLE_GENERAL)

        admin = Admin()
        admin.aname = 'test_name'
        admin.save()

        data = {
            'password1': 'password',
            'password2': 'password'
        }

        response = self.client.post(
            reverse(
                'admins.views.change_password',
                args=(admin.pk,)
            ), data
        )

        self.assertRedirects(response, reverse('admins.views.admin_list'))

        admin2 = Admin.objects.get(aname=admin.aname)

        self.assertEqual(admin, admin2)

        self.assertEqual(admin2.passwd, data['password1'])


class DeleteAdminViewTestCase(BaseViewTestCase):
    def test_delete_admin_for_custom_forbidden(self):
        self.switch_current_admin_role(settings.ROLE_CUSTOM)

        new_admin_pk = 9999

        response = self.client.get(
            reverse('admins.views.delete_admin', args=(new_admin_pk,))
        )

        self.assertEqual(response.status_code, httplib.FORBIDDEN)

    def test_delete_admin_for_general_not_found(self):
        self.switch_current_admin_role(settings.ROLE_GENERAL)

        not_exist_admin_pk = 9999

        response = self.client.get(
            reverse(
                'admins.views.delete_admin',
                args=(not_exist_admin_pk,)
            )
        )

        self.assertEqual(response.status_code, httplib.NOT_FOUND)

    def test_delete_admin_for_general_self_record(self):
        self.switch_current_admin_role(settings.ROLE_GENERAL)

        response = self.client.get(
            reverse(
                'admins.views.delete_admin',
                args=(self.admin_pk,)
            )
        )
        self.assertEqual(response.status_code, httplib.FORBIDDEN)

    def test_delete_admin_for_general_other_admin(self):
        self.switch_current_admin_role(settings.ROLE_GENERAL)

        new_admin = Admin()
        new_admin.save()

        new_admin_pk = new_admin.pk

        self.assertEqual(new_admin, Admin.objects.get(pk=new_admin_pk))

        response = self.client.get(
            reverse(
                'admins.views.delete_admin',
                args=(new_admin_pk,)
            )
        )

        self.assertRedirects(response, reverse('admins.views.admin_list'))

        self.assertRaises(
            Admin.DoesNotExist, Admin.objects.get, pk=new_admin_pk
        )
