
#
# Copyright (c) 2011, MegaVersiya, LTD. All rights reserved.
#
# Redistribution and use in source and binary forms, with or
# without modification, are permitted provided that
# the following conditions are met:
# Redistributions of source code must retain the above copyright notice,
# this list of conditions and the following disclaimer.
# Redistributions in binary form must reproduce the above copyright notice,
# this list of conditions and the following disclaimer in the documentation
# and/or other materials provided with the distribution.
# Neither the name of the MegaVersiya, LTD nor the names of its contributors
# may be used to endorse or promote products derived from this software
# without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
# OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
# OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT * LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
# OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
# DAMAGE.
#

import httplib

from django.test import TestCase
from django.core.urlresolvers import reverse
from django.conf import settings
from django.utils.translation import ugettext_lazy as _

from utils.test import BaseViewTestCase
from roles.models import Category
from categories.forms import CategoryForm


class NoAuthCategoryViewTestCase(TestCase):
    def test_admin_list(self):
        response = self.client.get(
            reverse('categories.views.category_list')
        )

        self.assertRedirects(response, settings.LOGIN_URL)

    def test_add_category(self):
        response = self.client.get(
            reverse('categories.views.add_category')
        )

        self.assertRedirects(response, settings.LOGIN_URL)

    def test_edit_category(self):
        response = self.client.get(
            reverse('categories.views.edit_category', args=(1,))
        )

        self.assertRedirects(response, settings.LOGIN_URL)

    def test_delete_category(self):
        response = self.client.get(
            reverse('categories.views.delete_category', args=(1,))
        )

        self.assertRedirects(response, settings.LOGIN_URL)


class CategoryListViewTestCase(BaseViewTestCase):
    def test_admin_list_for_general(self):
        response = self.client.get(
            reverse('categories.views.category_list')
        )

        self.assertEqual(response.status_code, httplib.OK)
        self.assertTemplateUsed(response, 'categories/category_list.html')

        self.assertTrue('category_list' in response.context)


class AddCategoryViewTestCase(BaseViewTestCase):
    def test_add_category(self):
        response = self.client.get(
            reverse('categories.views.add_category')
        )

        self.assertEqual(response.status_code, httplib.OK)

        self.assertTemplateUsed(response, 'categories/add_category.html')
        self.assertTemplateUsed(response, 'categories/category_form.html')

        self.assertTrue('form' in response.context)
        self.assertEqual(type(response.context['form']), CategoryForm)

    def test_add_category_send_empty_data(self):
        data = {
            'cname': ''
        }

        response = self.client.post(
            reverse('categories.views.add_category'), data
        )

        self.assertEqual(response.status_code, httplib.OK)

        self.assertTemplateUsed(response, 'categories/add_category.html')
        self.assertTemplateUsed(response, 'categories/category_form.html')

        self.assertTrue('form' in response.context)
        self.assertEqual(type(response.context['form']), CategoryForm)

        self.assertFormError(
            response, 'form', 'cname',
            _(u'This field is required.')
        )

    def test_add_category_send_success(self):
        data = {
            'cname': u'New category'
        }

        response = self.client.post(
            reverse('categories.views.add_category'), data
        )

        self.assertRedirects(
            response, reverse('categories.views.category_list')
        )

        category = Category.objects.get(cname=data['cname'])

        self.assertEqual(category.cname, data['cname'])
        self.assertEqual(category.type, settings.ROLE_CUSTOM)


class EditCategoryViewTestCase(BaseViewTestCase):
    def test_edit_category_not_found(self):
        response = self.client.get(
            reverse('categories.views.edit_category', args=(1,))
        )

        self.assertEqual(response.status_code, httplib.NOT_FOUND)

    def test_edit_category_general_forbidden(self):
        category = Category(
            cname=u'New category',
            type=settings.ROLE_GENERAL
        )
        category.save()

        response = self.client.get(
            reverse('categories.views.edit_category', args=(category.pk,))
        )

        self.assertEqual(response.status_code, httplib.FORBIDDEN)

    def test_edit_category(self):
        category = Category(
            cname=u'New category',
            type=settings.ROLE_CUSTOM
        )
        category.save()

        response = self.client.get(
            reverse('categories.views.edit_category', args=(category.pk,))
        )

        self.assertEqual(response.status_code, httplib.OK)

        self.assertTemplateUsed(response, 'categories/edit_category.html')
        self.assertTemplateUsed(response, 'categories/category_form.html')

        self.assertTrue('form' in response.context)
        self.assertEqual(type(response.context['form']), CategoryForm)

    def test_edit_category_send_empty_data(self):
        category = Category(
            cname=u'New category',
            type=settings.ROLE_CUSTOM
        )
        category.save()

        data = {
            'cname': ''
        }

        response = self.client.post(
            reverse('categories.views.edit_category', args=(category.pk,)),
            data
        )

        self.assertEqual(response.status_code, httplib.OK)

        self.assertTemplateUsed(response, 'categories/edit_category.html')
        self.assertTemplateUsed(response, 'categories/category_form.html')

        self.assertTrue('form' in response.context)
        self.assertEqual(type(response.context['form']), CategoryForm)

        self.assertFormError(
            response, 'form', 'cname',
            _(u'This field is required.')
        )

    def test_edit_category_send_success(self):
        category = Category(
            cname=u'New category',
            type=settings.ROLE_CUSTOM
        )
        category.save()

        data = {
            'cname': u'New category renamed'
        }

        response = self.client.post(
            reverse('categories.views.edit_category', args=(category.pk,)),
            data
        )

        self.assertRedirects(
            response, reverse('categories.views.category_list')
        )

        category = Category.objects.get(cname=data['cname'])

        self.assertEqual(category.cname, data['cname'])
        self.assertEqual(category.type, settings.ROLE_CUSTOM)


class DeleteCategoryViewTestCase(BaseViewTestCase):
    def test_delete_category_not_found(self):
        response = self.client.get(
            reverse('categories.views.delete_category', args=(1,))
        )

        self.assertEqual(response.status_code, httplib.NOT_FOUND)

    def test_delete_category_general_forbidden(self):
        category = Category(
            cname=u'New category',
            type=settings.ROLE_GENERAL
        )
        category.save()

        response = self.client.get(
            reverse('categories.views.delete_category', args=(category.pk,))
        )

        self.assertEqual(response.status_code, httplib.FORBIDDEN)

    def test_delete_category_custom_success(self):
        category = Category(
            cname=u'New category',
            type=settings.ROLE_CUSTOM
        )
        category.save()

        category_pk = category.pk

        self.assertEqual(category, Category.objects.get(pk=category_pk))

        response = self.client.get(
            reverse('categories.views.delete_category', args=(category_pk,))
        )

        self.assertRedirects(
            response, reverse('categories.views.category_list')
        )

        self.assertRaises(
            Category.DoesNotExist, Category.objects.get, pk=category_pk
        )
