
#
# Copyright (c) 2011, MegaVersiya, LTD. All rights reserved.
#
# Redistribution and use in source and binary forms, with or
# without modification, are permitted provided that
# the following conditions are met:
# Redistributions of source code must retain the above copyright notice,
# this list of conditions and the following disclaimer.
# Redistributions in binary form must reproduce the above copyright notice,
# this list of conditions and the following disclaimer in the documentation
# and/or other materials provided with the distribution.
# Neither the name of the MegaVersiya, LTD nor the names of its contributors
# may be used to endorse or promote products derived from this software
# without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
# OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
# OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT * LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
# OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
# DAMAGE.
#

from django.shortcuts import render, get_object_or_404, redirect
from django.core.urlresolvers import reverse
from django.core.context_processors import csrf
from django.conf import settings
from django.utils.translation import ugettext as _
from django.contrib import messages

from core.http import Http403
from auth.decorators import login_required
from roles.models import Category
from categories.forms import CategoryForm


@login_required
def category_list(request):
    """All category list."""
    context = {
        'category_list': Category.objects.all()
    }

    return render(request, 'categories/category_list.html', context)


@login_required
def add_category(request):
    """Add new category."""
    if request.method == 'POST':
        form = CategoryForm(request.POST)
        if form.is_valid():
            max_number = Category.objects.get_max_cat_number()
            cat_number = max_number + 1000

            category = form.save(commit=False)
            category.cat = cat_number
            category.save()

            messages.success(
                request,
                _(u'Category was successfully added.')
            )

            return redirect(reverse('categories.views.category_list'))
    else:
        form = CategoryForm()

    context = {
        'form': form
    }
    context.update(csrf(request))

    return render(request, 'categories/add_category.html', context)


@login_required
def edit_category(request, category_pk):
    """Edit one category by pk."""
    category = get_object_or_404(Category, pk=category_pk)

    if category.type != settings.ROLE_CUSTOM:
        raise Http403

    if request.method == 'POST':
        form = CategoryForm(request.POST, instance=category)

        if form.is_valid():
            form.save()

            messages.success(
                request,
                _(u'Category was successfully edited.')
            )

            return redirect(reverse('categories.views.category_list'))
    else:
        form = CategoryForm(instance=category)

    context = {
        'category': category,
        'form': form
    }
    context.update(csrf(request))

    return render(request, 'categories/edit_category.html', context)


@login_required
def delete_category(request, category_pk):
    """Delete one category by pk."""
    category = get_object_or_404(Category, pk=category_pk)

    if category.type != settings.ROLE_CUSTOM:
        raise Http403

    category.delete()

    messages.success(request, _(u'Category was successfully deleted.'))

    return redirect(reverse('categories.views.category_list'))
