
#
# Copyright (c) 2011, MegaVersiya, LTD. All rights reserved.
#
# Redistribution and use in source and binary forms, with or
# without modification, are permitted provided that
# the following conditions are met:
# Redistributions of source code must retain the above copyright notice,
# this list of conditions and the following disclaimer.
# Redistributions in binary form must reproduce the above copyright notice,
# this list of conditions and the following disclaimer in the documentation
# and/or other materials provided with the distribution.
# Neither the name of the MegaVersiya, LTD nor the names of its contributors
# may be used to endorse or promote products derived from this software
# without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
# OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
# OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT * LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
# OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
# DAMAGE.
#

from django.shortcuts import render, get_object_or_404, redirect
from django.core.urlresolvers import reverse
from django.core.context_processors import csrf
from django.core.cache import cache
from django.conf import settings
from django.utils.translation import ugettext as _
from django.contrib import messages

from auth.decorators import login_required
from roles.models import Category
from roles.forms import CategoryListForm
from manual_categorization.lib import ZoneFileWriter
from manual_categorization.models import Domain, DomainCategory
from manual_categorization.forms import DomainForm


def write_zone_file():
    buffer_size = 1024 * 1024 * 10

    writer = ZoneFileWriter(
        path=settings.DNS_ZONE_PATH,
        ttl=settings.DNS_TTL,
        buffer_size=buffer_size,
    )

    domains = Domain.objects.all()
    for domain in domains:
        category_list = domain.categories.all()
        if category_list:
            writer.write(
                domain.dname, [cat.pk for cat in category_list]
            )

    writer.flush()


@login_required
def domain_list(request):
    """All domain list."""
    context = {
        'domain_list': Domain.objects.all()
    }

    return render(
        request, 'manual_categorization/domain_list.html', context
    )


@login_required
def add_domain(request):
    """Add new domain."""
    category_list = Category.objects.all()

    if request.method == 'POST':
        domain_form = DomainForm(request.POST)

        category_list_form = CategoryListForm(
            category_list, request.POST
        )

        if domain_form.is_valid() and category_list_form.is_valid():

            domain = domain_form.save()

            for key in category_list_form.changed_data:
                if key.startswith('category'):
                    cat_checked = category_list_form.cleaned_data[key]
                    cat_id = int(key.split('_')[1])

                    DomainCategory.objects.create_or_delete(
                       cat_checked, domain, cat_id
                    )

            write_zone_file()

            cache.clear()

            messages.success(
                request,
                _(u'Domain was successfully added.')
            )

            return redirect(
                reverse('manual_categorization.views.domain_list')
            )
    else:
        domain_form = DomainForm()

        category_list_form = CategoryListForm(category_list)

    context = {
        'domain_form': domain_form,
        'category_list_form': category_list_form
    }
    context.update(csrf(request))

    return render(
        request, 'manual_categorization/add_domain.html', context
    )


@login_required
def edit_domain(request, domain_pk):
    """Edit one domain by pk."""
    domain = get_object_or_404(Domain, pk=domain_pk)

    category_list = Category.objects.all()

    object_list = DomainCategory.objects.get_by_domain(domain)

    initial = {}
    for dc in object_list:
        initial['category_%d' % dc.cat.cat] = True

    if request.method == 'POST':
        domain_form = DomainForm(request.POST, instance=domain)

        category_list_form = CategoryListForm(
            category_list, request.POST, initial=initial
        )

        if domain_form.is_valid() and category_list_form.is_valid():

            domain = domain_form.save()

            for key in category_list_form.changed_data:
                if key.startswith('category'):
                    cat_checked = category_list_form.cleaned_data[key]
                    cat_id = int(key.split('_')[1])

                    DomainCategory.objects.create_or_delete(
                       cat_checked, domain, cat_id
                    )

            write_zone_file()

            cache.clear()

            messages.success(
                request,
                _(u'Domain was successfully edited.')
            )

            return redirect(
                reverse('manual_categorization.views.domain_list')
            )
    else:
        domain_form = DomainForm(instance=domain)

        category_list_form = CategoryListForm(category_list, initial=initial)

    context = {
        'domain_form': domain_form,
        'category_list_form': category_list_form
    }
    context.update(csrf(request))

    return render(
        request, 'manual_categorization/edit_domain.html', context
    )


@login_required
def delete_domain(request, domain_pk):
    """Delete one domain by pk."""
    domain = get_object_or_404(Domain, pk=domain_pk)
    domain.delete()

    write_zone_file()

    cache.clear()

    messages.success(request, _(u'Domain was successfully deleted.'))

    return redirect(
        reverse('manual_categorization.views.domain_list')
    )
