
#
# Copyright (c) 2011, MegaVersiya, LTD. All rights reserved.
#
# Redistribution and use in source and binary forms, with or
# without modification, are permitted provided that
# the following conditions are met:
# Redistributions of source code must retain the above copyright notice,
# this list of conditions and the following disclaimer.
# Redistributions in binary form must reproduce the above copyright notice,
# this list of conditions and the following disclaimer in the documentation
# and/or other materials provided with the distribution.
# Neither the name of the MegaVersiya, LTD nor the names of its contributors
# may be used to endorse or promote products derived from this software
# without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
# OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
# OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT * LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
# OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
# DAMAGE.
#

from django.db import models, connection
from django.conf import settings


def dictfetchall(cursor):
    "Returns all rows from a cursor as a dictionary."
    desc = cursor.description

    return [
        dict(zip([col[0] for col in desc], row))
        for row in cursor.fetchall()
    ]


class RoleManager(models.Manager):
    """Manager for Role model."""
    def get_by_type(self, action_type):
        """Return all roles with given type."""
        return self.filter(type=action_type).all()

    def get_all_join_parents(self):
        role_list = self.all()

        parents = Hierarchy.objects.get_parents_join_roles()

        for role in role_list:
            role.parents = []
            for parent in parents:
                self_ref = (parent[u'child'] == parent[u'parent'])
                if (role.pk == parent[u'child']) and not self_ref:
                    role.parents.append(parent)

        return role_list


class Role(models.Model):
    """User Role model."""
    rolid = models.AutoField(primary_key=True)
    rname = models.CharField(unique=True, max_length=40)
    type = models.CharField(max_length=1, choices=settings.ROLE_TYPES)

    objects = RoleManager()

    class Meta:
        db_table = u'roles'

    def __unicode__(self):
        return self.rname


class ListManager(models.Manager):
    """Manager for List model."""
    def get_for_role(self, role, action):
        """Return all lists for given role and type."""
        return self.filter(
            rolid=role,
            action=action
        ).order_by(u'url').all()

    def get_for_role_pks(self, role_pks, action):
        """Return all lists for given role pks list and type."""
        return self.filter(
            rolid__in=role_pks,
            action=action
        ).order_by(u'url').all()


class List(models.Model):
    """Special (white, black etc) List model."""
    lstid = models.AutoField(primary_key=True)
    rolid = models.ForeignKey(Role, db_column=u'rolid')
    url = models.CharField(max_length=200)
    stars = models.BooleanField()
    action = models.IntegerField(choices=settings.ACTION_TYPES)

    objects = ListManager()

    class Meta:
        db_table = u'lists'


class HierarchyManager(models.Manager):
    """Manager for Hierarchy model."""
    def child_count_for_role(self, role):
        """Return count of child roles."""
        return self.filter(
            parent=role.pk
        ).exclude(child=role.pk).count()

    def get_parents_join_roles(self):
        """Return list of parent roles for given role."""
        cursor = connection.cursor()
        cursor.execute(
            """SELECT h.child, h.parent, r.rname
               FROM hierarchy h
               INNER JOIN roles r
               ON h.parent=r.rolid;
            """
        )

        return dictfetchall(cursor)

    def get_parent_pks_for_role(self, role):
        """
        Return list of parent roles primary keys for given role.
        """
        return self.filter(
            child=role.pk
        ).values_list(u'parent', flat=True)

    def create_or_delete(self, is_checked, role_pk, item_role_pk):
        """
        Delete hierarchy record if exists or create if not.
        """
        if is_checked:
            hierarchy_item = Hierarchy(
                parent=role_pk,
                child=item_role_pk
            )
            hierarchy_item.save()
        else:
            try:
                hierarchy_item = Hierarchy.objects.get(
                    parent=role_pk,
                    child=item_role_pk
                )
                hierarchy_item.delete()
            except Hierarchy.DoesNotExist:
                pass


class Hierarchy(models.Model):
    """Roles parent-child Hierarchy model."""
    parent = models.PositiveIntegerField(null=False, blank=False)
    child = models.PositiveIntegerField(null=False, blank=False)

    objects = HierarchyManager()

    class Meta:
        db_table = u'hierarchy'
        unique_together = (u'parent', u'child')


class CategoryManager(models.Manager):
    """Manager for Category model."""
    def get_max_cat_number(self):
        """Return max stored category number."""
        result = self.aggregate(models.Max(u'cat'))[u'cat__max']

        if not result:
            return 0

        return result

    def as_dict(self):
        return dict(self.values_list(u'cat', u'cname'))


class Category(models.Model):
    """Resource category model."""
    cat = models.AutoField(primary_key=True)
    cname = models.CharField(null=False, blank=False, max_length=255)
    type = models.CharField(
        max_length=1,
        choices=settings.ROLE_TYPES,
        default=settings.ROLE_CUSTOM
    )

    objects = CategoryManager()

    class Meta:
        db_table = u'categories'
        ordering = [u'cname']

    def __unicode__(self):
        return self.cname


class PermManager(models.Manager):
    """Manager for Perm model."""
    def get_for_role_and_action(self, role, action):
        """Return category permissions for given role and action."""
        return self.filter(
            rolid=role,
            action=action
        ).order_by(u'cat').all()

    def get_for_pks_and_action(self, role_pks, action):
        """
        Return category permissions for given role's pks list and action.
        """
        return self.filter(
            rolid__in=role_pks,
            action=action
        ).select_related().order_by(u'cat').distinct().all()

    def create_or_delete(self, is_checked, role, cat_id, action):
        """
        Delete permission record if exists or create if not.
        """
        cat = Category.objects.get(pk=cat_id)

        if is_checked:
            p = Perm(rolid=role, cat=cat, action=action)
            p.save()
        else:
            try:
                p = Perm.objects.get(
                    rolid=role,
                    cat=cat,
                    action=action
                )
                p.delete()
            except Perm.DoesNotExist:
                pass


class Perm(models.Model):
    """Category permissions for role model."""
    prmid = models.AutoField(primary_key=True)
    rolid = models.ForeignKey(Role, db_column=u'rolid')
    cat = models.ForeignKey(Category, db_column=u'cat')
    action = models.IntegerField(choices=settings.ACTION_TYPES)

    objects = PermManager()

    class Meta:
        db_table = u'perms'
