
#
# Copyright (c) 2011, MegaVersiya, LTD. All rights reserved.
#
# Redistribution and use in source and binary forms, with or
# without modification, are permitted provided that
# the following conditions are met:
# Redistributions of source code must retain the above copyright notice,
# this list of conditions and the following disclaimer.
# Redistributions in binary form must reproduce the above copyright notice,
# this list of conditions and the following disclaimer in the documentation
# and/or other materials provided with the distribution.
# Neither the name of the MegaVersiya, LTD nor the names of its contributors
# may be used to endorse or promote products derived from this software
# without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
# OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
# OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT * LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
# OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
# DAMAGE.
#

from django.shortcuts import render, get_object_or_404, redirect
from django.http import Http404
from django.core.urlresolvers import reverse
from django.core.context_processors import csrf
from django.utils.translation import ugettext as _
from django.conf import settings
from django.core.cache import cache
from django.contrib import messages

from core.http import Http403
from auth.decorators import login_required
from users.models import User
from roles.models import Role, Hierarchy, Perm, List, Category
from roles.forms import RoleForm, ParentRolesForm
from roles.forms import AddListItemForm, CategoryListForm


@login_required
def role_list(request):
    """Display all roles list."""
    current_admin = request.session['current_admin']

    context = {
        'role_list': Role.objects.get_all_join_parents(),
        'editable_role_type': current_admin.type
    }

    return render(request, 'roles/role_list.html', context)


@login_required
def add_role(request):
    """Add new role."""
    current_admin = request.session['current_admin']

    if current_admin.type == settings.ROLE_GENERAL:
        possible_parent_roles = Role.objects.get_by_type(
            settings.ROLE_GENERAL
        )
    else:
        possible_parent_roles = Role.objects.all()

    if request.method == 'POST':
        role_form = RoleForm(request.POST)

        parent_roles_form = ParentRolesForm(
            possible_parent_roles,
            current_admin.type,
            request.POST
        )

        if role_form.is_valid() and parent_roles_form.is_valid():

            role = role_form.save(commit=False)
            role.type = current_admin.type
            role.save()

            hierarchy_item = Hierarchy(parent=role.pk, child=role.pk)
            hierarchy_item.save()

            for key in parent_roles_form.changed_data:
                if key.startswith('role'):
                    role_checked = parent_roles_form.cleaned_data[key]
                    parent_role_pk = int(key.split('_')[1])

                    Hierarchy.objects.create_or_delete(
                        role_checked, parent_role_pk, role.pk
                    )

            messages.success(request, _(u'Role was successfully added.'))

            return redirect(reverse('roles.views.role_list'))
    else:
        role_form = RoleForm()
        parent_roles_form = ParentRolesForm(
            possible_parent_roles, current_admin.type
        )

    context = {
        'role_form': role_form,
        'parent_roles_form': parent_roles_form
    }
    context.update(csrf(request))

    return render(request, 'roles/add_role.html', context)


@login_required
def view_role(request, pk):
    """View one role properties."""
    role = get_object_or_404(Role, pk=pk)

    parent_pks = Hierarchy.objects.get_parent_pks_for_role(role)

    white_list = List.objects.get_for_role_pks(
        parent_pks, settings.ACTION_ACCEPT
    )
    black_list = List.objects.get_for_role_pks(
        parent_pks,
        settings.ACTION_REJECT
    )

    accepted_cat_list = Perm.objects.get_for_pks_and_action(
        parent_pks, settings.ACTION_ACCEPT
    )

    rejected_cat_list = Perm.objects.get_for_pks_and_action(
        parent_pks, settings.ACTION_REJECT
    )

    redirected_cat_list = Perm.objects.get_for_pks_and_action(
        parent_pks, settings.ACTION_REDIRECT
    )

    context = {
        'role': role,
        'white_list': white_list,
        'black_list': black_list,
        'accepted_cat_list': accepted_cat_list,
        'rejected_cat_list': rejected_cat_list,
        'redirected_cat_list': redirected_cat_list
    }

    return render(request, 'roles/view_role.html', context)


@login_required
def edit_role(request, pk):
    """Edit one role."""
    current_admin = request.session['current_admin']

    role = get_object_or_404(Role, pk=pk)

    if current_admin.type != role.type:
        raise Http403

    if request.method == 'POST':
        form = RoleForm(request.POST, instance=role)
        if form.is_valid():
            form.save()

            cache.clear()

            messages.success(
                request, _(u'Role was successfully edited.')
            )

            return redirect(reverse('roles.views.role_list'))
    else:
        form = RoleForm(instance=role)

    context = {
        'role': role,
        'form': form
    }
    context.update(csrf(request))

    return render(request, 'roles/edit_role.html', context)


@login_required
def delete_role(request, pk):
    """Delete one role  by pk."""
    current_admin = request.session['current_admin']

    role = get_object_or_404(Role, pk=pk)

    if current_admin.type != role.type:
        raise Http403

    user_count = User.objects.user_count_with_role(role)

    if user_count > 0:
        messages.error(
            request,
            _(u'There is at least one user associated with the role.')
        )
    else:
        child_role_count = Hierarchy.objects.child_count_for_role(role)
        if child_role_count > 0:
            messages.error(
                request, _(u'Role is a parent for another role.')
            )
        else:
            role.delete()

            cache.clear()

            messages.success(request, _(u'Role was successfully deleted.'))

    return redirect(reverse('roles.views.role_list'))


@login_required
def url_list(request, role_pk, list_type):
    """Edit lists (white and black)."""
    current_admin = request.session['current_admin']

    role = get_object_or_404(Role, pk=role_pk)

    if current_admin.type != role.type:
        raise Http403

    if list_type == 'white':
        action = settings.ACTION_ACCEPT
    elif list_type == 'black':
        action = settings.ACTION_REJECT
    else:
        raise Http404

    object_list = List.objects.get_for_role(role, action)

    def to_regex(url, exact_match):
        """Build regex pattern from url."""
        url = '^' + url.replace('.', r'\.')
        if not exact_match:
            url += '.*'
        else:
            if not url.endswith('/'):
                url += '/'
        url += '$'

        return url

    if request.method == 'POST':
        form = AddListItemForm(
            request.POST,
            initial={'rolid': role, 'action': action}
        )

        if form.is_valid():
            exact_match = int(form.cleaned_data['exact_match'])
            url = to_regex(form.cleaned_data['url'], exact_match)

            list_item = List(
                rolid=role,
                url=url,
                stars=(not exact_match),
                action=form.cleaned_data['action']
            )
            list_item.save()

            cache.clear()

            messages.success(
                request,
                _(u'List item was successfully added.')
            )

            return redirect(
                reverse('roles.views.edit_role', args=(role_pk,))
            )
    else:
        form = AddListItemForm(
            initial={'rolid': role.pk, 'action': action}
        )

    context = {
        'object_list': object_list,
        'role': role,
        'action': action,
        'form': form
    }
    context.update(csrf(request))

    return render(request, 'roles/edit_list.html', context)


@login_required
def delete_url_list_item(request, role_pk, list_pk):
    """Delete one list item by pk."""
    current_admin = request.session['current_admin']

    role = get_object_or_404(Role, pk=role_pk)

    if current_admin.type != role.type:
        raise Http403

    list_item = get_object_or_404(List, pk=list_pk)
    list_item.delete()

    cache.clear()

    messages.success(request, _(u'List item was successfully deleted.'))

    return redirect(
        reverse('roles.views.edit_role', args=(role_pk,))
    )


@login_required
def categories(request, role_pk, category_type):
    """Edit categories list for given role and type."""
    current_admin = request.session['current_admin']

    role = get_object_or_404(Role, pk=role_pk)

    if current_admin.type != role.type:
        raise Http403

    type_to_action = {
        'accepted': settings.ACTION_ACCEPT,
        'rejected': settings.ACTION_REJECT,
        'redirected': settings.ACTION_REDIRECT
    }

    if current_admin.type == settings.ROLE_CUSTOM:
        if category_type == 'accepted':
            raise Http403

    try:
        action = type_to_action[category_type]
    except KeyError:
        raise Http404

    context = {
        'role': role,
        'action': action
    }
    context.update(csrf(request))

    category_list = Category.objects.all()

    object_list = Perm.objects.get_for_role_and_action(role, action)

    initial = {}
    for perm in object_list:
        initial['category_%d' % perm.cat.pk] = True

    if request.method == 'POST':
        form = CategoryListForm(
            category_list, request.POST, initial=initial
        )

        if form.is_valid():
            for key in form.changed_data:
                if key.startswith('category'):
                    cat_checked = form.cleaned_data[key]
                    cat_id = int(key.split('_')[1])

                    Perm.objects.create_or_delete(
                        cat_checked, role, cat_id, action
                    )

            cache.clear()

            messages.success(
                request, _(u'Categories was successfully saved.')
            )

            return redirect(
                reverse('roles.views.edit_role', args=(role_pk,))
            )
    else:
        form = CategoryListForm(category_list, initial=initial)

    context['form'] = form

    return render(request, 'roles/categories.html', context)


@login_required
def parent_roles(request, role_pk):
    """Edit parent roles list."""
    current_admin = request.session['current_admin']

    role = get_object_or_404(Role, pk=role_pk)

    if current_admin.type == settings.ROLE_GENERAL:
        possible_parent_roles = Role.objects.get_by_type(
            settings.ROLE_GENERAL
        )
    else:
        possible_parent_roles = Role.objects.all()

    possible_parent_roles = [
        item for item in possible_parent_roles if item.pk != role.pk
    ]

    initial = {}

    parent_pks = Hierarchy.objects.get_parent_pks_for_role(role)

    for pk in parent_pks:
        initial['role_%d' % pk] = True

    if request.method == 'POST':
        form = ParentRolesForm(
            possible_parent_roles,
            current_admin.type,
            request.POST
        )

        if form.is_valid():
            for key in form.changed_data:
                if key.startswith('role'):
                    role_checked = form.cleaned_data[key]
                    parent_role_pk = int(key.split('_')[1])

                    Hierarchy.objects.create_or_delete(
                        role_checked, parent_role_pk, role_pk
                    )

            cache.clear()

            messages.success(
                request,
                _(u'Parent roles were successfully edited.')
            )

            return redirect(
                reverse('roles.views.edit_role', args=(role_pk,))
            )
    else:
        form = ParentRolesForm(
            possible_parent_roles,
            current_admin.type,
            initial=initial
        )

    context = {
        'role': role,
        'form': form
    }
    context.update(csrf(request))

    return render(request, 'roles/parent_roles.html', context)
