
#
# Copyright (c) 2011, MegaVersiya, LTD. All rights reserved.
#
# Redistribution and use in source and binary forms, with or
# without modification, are permitted provided that
# the following conditions are met:
# Redistributions of source code must retain the above copyright notice,
# this list of conditions and the following disclaimer.
# Redistributions in binary form must reproduce the above copyright notice,
# this list of conditions and the following disclaimer in the documentation
# and/or other materials provided with the distribution.
# Neither the name of the MegaVersiya, LTD nor the names of its contributors
# may be used to endorse or promote products derived from this software
# without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
# OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
# OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT * LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
# OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
# DAMAGE.
#

import time
from datetime import date

from django.conf import settings


class FileProvider(object):
    """
    Return data from file-like object,
    one dict per iteration.
    """
    def __init__(
                  self,
                  source,
                  delimiter=' ',
                  start_position=0,
                  debug=False):
        if isinstance(source, basestring):
            self.source = open(source)
        else:
            self.source = source

        self.delimiter = delimiter
        self.start_position = start_position
        self.debug = debug

        self.rewind()

    def __iter__(self):
        return self

    def next(self):
        while True:
            raw_line = self.source.readline()
            if not raw_line:
                self.rewind()

                raise StopIteration

            chunks = raw_line.strip().split(self.delimiter)

            try:
                return self._process(chunks)
            except:
                if self.debug:
                    raise

    def rewind(self):
        self.source.seek(self.start_position)

    def _process(self, result):
        raise NotImplementedError

    def tell(self):
        return self.source.tell()


class CustomLogProvider(FileProvider):
    """
    Return data from log file,
    one dictionary per iteration.
    """
    def __init__(self, source, *args, **kwargs):
        super(CustomLogProvider, self).__init__(source, *args, **kwargs)

        self.keys = (
            'timestamp', 'mod', 'user', 'client_ip', 'url',
            'role', 'categories', 'action'
        )

    def _process(self, chunks):
        # 1. Chunk count control.
        if len(chunks) != len(self.keys):
            raise ValueError

        result = {}
        for i, key in enumerate(self.keys):
            result[key] = chunks[i]

        # 2. Symbol name to number.
        action = 'ACTION_' + result['action']
        result['action'] = getattr(settings, action, -1)

        # 3. Force to unicode, skip on exception.
        result['url'] = unicode(result['url'])

        # 4. Save date version for every timestamp.
        result['date'] = date.fromtimestamp(float(result['timestamp']))

        return result
