
#
# Copyright (c) 2011, MegaVersiya, LTD. All rights reserved.
#
# Redistribution and use in source and binary forms, with or
# without modification, are permitted provided that
# the following conditions are met:
# Redistributions of source code must retain the above copyright notice,
# this list of conditions and the following disclaimer.
# Redistributions in binary form must reproduce the above copyright notice,
# this list of conditions and the following disclaimer in the documentation
# and/or other materials provided with the distribution.
# Neither the name of the MegaVersiya, LTD nor the names of its contributors
# may be used to endorse or promote products derived from this software
# without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
# OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
# OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT * LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
# OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
# DAMAGE.
#

import datetime
import time

from django.http import HttpResponse, Http404
from django.shortcuts import render
from django.core.context_processors import csrf
from django.conf import settings
from django.utils.datastructures import SortedDict
from django.utils.encoding import force_unicode
from django.views.decorators.cache import never_cache
from django.core.paginator import Paginator, EmptyPage, PageNotAnInteger
from pyofc2 import title, bar, open_flash_chart
from pyofc2 import x_axis, y_axis, x_axis_labels

from auth.decorators import login_required
from roles.models import Category
from statistics.models import StatisticsItem
from statistics.forms import FilterForm, TABLE_FORMAT, CHART_FORMAT


def _get_condiotions(filter):
    """Return conditions dictionary for queryset filter method."""
    conditions = {}

    user = filter['user']
    if user:
        equals_to = u'' if user.is_default() else force_unicode(user)
        conditions['user'] = equals_to

    role = filter['role']
    if role:
        conditions['role'] = force_unicode(role)

    from_date = filter['from_date']
    if from_date:
        conditions['date__gte'] = from_date

    to_date = filter['to_date']
    if to_date:
        tomorrow = to_date + datetime.timedelta(days=1)
        conditions['date__lte'] = tomorrow

    category = filter['category']
    if category:
        pattern = '(^%s$)|(^%s:[\d:]+$)|(^[\d:]+:%s$)'.replace(
            '%s', force_unicode(category.cat)
        )
        conditions['categories__regex'] = pattern

    action = filter['action']
    if action:
        conditions['action'] = action

    client_ip = filter['client_ip']
    if client_ip:
        conditions['client_ip__regex'] = client_ip

    url = filter['url']
    if url:
        conditions['url__regex'] = url

    return conditions


def _get_table_context(queryset, filter, request):
    items_per_page = int(filter['items_per_page'])

    page = request.GET.get('page', 1)

    paginator = Paginator(queryset, items_per_page)

    try:
        page_object = paginator.page(page)
    except EmptyPage:
        page_object = paginator.page(paginator.num_pages)

    return {
        'page_object': page_object,
        'category_dict': Category.objects.as_dict()
    }


def _get_chart_context(queryset, filter):
    data_to_chart = SortedDict()

    from_date = filter['from_date']
    to_date = filter['to_date']

    current_date = from_date
    while current_date <= to_date:
        data_to_chart[current_date.strftime('%Y-%m-%d')] = 0
        current_date += datetime.timedelta(days=1)

    for item in queryset:
        str_date = item.date.strftime('%Y-%m-%d')
        if item.date and (str_date in data_to_chart):
            data_to_chart[str_date] += 1

    return {
        'data_to_chart': data_to_chart,
        'chart_title': '%s - %s' % (from_date, to_date),
    }


@login_required
def main(request):
    """Display system usage statistics."""
    context = {
        'CHART_FORMAT': CHART_FORMAT,
        'TABLE_FORMAT': TABLE_FORMAT
    }

    statistics_filter = None

    if request.method == 'POST':
        form = FilterForm(request.POST)

        if form.is_valid():
            statistics_filter = form.cleaned_data

            request.session['statistics_filter'] = statistics_filter
    else:
        statistics_filter = request.session.get('statistics_filter')

        form = FilterForm(statistics_filter)

    if statistics_filter:
        statistics_queryset = StatisticsItem.objects.filter(
            **_get_condiotions(statistics_filter)
        ).order_by('date')

        if statistics_filter['format'] == TABLE_FORMAT:
            context.update(_get_table_context(
                statistics_queryset, statistics_filter, request)
            )
        elif statistics_filter['format'] == CHART_FORMAT:
            request.session['statistics'] = _get_chart_context(
                statistics_queryset, statistics_filter
            )

        context['display_format'] = statistics_filter['format']

    context['form'] = form
    context.update(csrf(request))

    return render(request, 'statistics/main.html', context)


@never_cache
@login_required
def chart_data(request):
    """Provide JSON encoded data for request chart."""

    statistics = request.session.get('statistics')
    if not statistics:
        raise Http404

    data_to_chart = statistics.get('data_to_chart')
    chart_title = statistics.get('chart_title', u'')

    if not data_to_chart:
        raise Http404

    max_value = max(data_to_chart.values())
    day_count = len(data_to_chart)

    # Build chart.

    def get_percent(value, percent):
        return int(float(value) / 100 * percent)

    chart_title = title(text=chart_title)

    theme_options = {
        'background_color': '#ffffff',
        'base_color': '#56acde',
        'tick_height': 2,
        'font_size': 8
    }

    b = bar(
        colour=theme_options['base_color'],
        values=data_to_chart.values(),
        outline_colour='#577261',
        font_size=theme_options['font_size']
    )

    xlbls = x_axis_labels(
        rotate='horizontal',
        colour=theme_options['tick_height'],
        labels=data_to_chart.keys()
    )

    x = x_axis(
        colour=theme_options['base_color'],
        tick_height=theme_options['tick_height'],
        max=day_count,
        labels=xlbls
    )

    y = y_axis(
        colour=theme_options['base_color'],
        tick_height=theme_options['tick_height'],
        steps=get_percent(max_value, 10),
        max=max_value + get_percent(max_value, 10)
    )

    chart = open_flash_chart(
        bg_colour=theme_options['background_color'],
        title=chart_title,
        x_axis=x,
        y_axis=y
    )
    chart.add_element(b)

    del request.session['statistics']

    return HttpResponse(chart.render(), mimetype='application/json')
