
#
# Copyright (c) 2011, MegaVersiya, LTD. All rights reserved.
#
# Redistribution and use in source and binary forms, with or
# without modification, are permitted provided that
# the following conditions are met:
# Redistributions of source code must retain the above copyright notice,
# this list of conditions and the following disclaimer.
# Redistributions in binary form must reproduce the above copyright notice,
# this list of conditions and the following disclaimer in the documentation
# and/or other materials provided with the distribution.
# Neither the name of the MegaVersiya, LTD nor the names of its contributors
# may be used to endorse or promote products derived from this software
# without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
# OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
# OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT * LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
# OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
# DAMAGE.
#

import httplib

from django.core.urlresolvers import reverse
from django.test import TestCase
from django.conf import settings

from utils.test import BaseViewTestCase
from roles.models import Role
from users.models import User
from users.forms import UserForm


class NoAuthUserViewTestCase(TestCase):
    def test_user_list(self):
        response = self.client.get(reverse('users.views.user_list'))

        self.assertRedirects(response, settings.LOGIN_URL)

    def test_add_user(self):
        response = self.client.get(reverse('users.views.add_user'))

        self.assertRedirects(response, settings.LOGIN_URL)

    def test_edit_user(self):
        response = self.client.get(reverse('users.views.edit_user', args=(1,)))

        self.assertRedirects(response, settings.LOGIN_URL)

    def test_delete(self):
        response = self.client.get(
            reverse('users.views.delete_user', args=(1,))
        )

        self.assertRedirects(response, settings.LOGIN_URL)


class UserListViewTestCase(BaseViewTestCase):
    def test_empty_user_list(self):
        response = self.client.get(reverse('users.views.user_list'))

        self.assertEqual(response.status_code, httplib.OK)
        self.assertTemplateUsed(response, 'users/user_list.html')

        self.assertEqual(len(response.context['user_list']), 0)

    def test_user_list_for_general(self):
        self.switch_current_admin_role(settings.ROLE_GENERAL)

        response = self.client.get(reverse('users.views.user_list'))

        self.assertFalse(response.context['can_add'])
        self.assertFalse(response.context['can_edit'])
        self.assertFalse(response.context['can_delete'])

    def test_user_list_for_custom(self):
        self.switch_current_admin_role(settings.ROLE_CUSTOM)

        response = self.client.get(reverse('users.views.user_list'))

        self.assertTrue(response.context['can_add'])
        self.assertTrue(response.context['can_edit'])
        self.assertTrue(response.context['can_delete'])

    def test_user_list_many_items(self):
        role = Role()
        role.rname = u'test_role'
        role.save()

        user_count = 3

        for i in xrange(user_count):
            user = User(
                rolid=role,
                uname=u'test_name_%d' % i
            )
            user.save()

        response = self.client.get(reverse('users.views.user_list'))

        self.assertEqual(
            len(response.context['user_list']),
            user_count
        )


class AddUserViewTestCase(BaseViewTestCase):
    def test_add_user_for_general(self):
        self.switch_current_admin_role(settings.ROLE_GENERAL)

        response = self.client.get(reverse('users.views.add_user'))
        self.assertEqual(response.status_code, httplib.FORBIDDEN)

    def test_add_user_for_custom(self):
        self.switch_current_admin_role(settings.ROLE_CUSTOM)

        response = self.client.get(reverse('users.views.add_user'))

        self.assertEqual(response.status_code, httplib.OK)
        self.assertTemplateUsed(response, 'users/add_user.html')

        self.assertEqual(type(response.context['form']), UserForm)

    def test_add_user_send_data_success(self):
        self.switch_current_admin_role(settings.ROLE_CUSTOM)

        role = Role(
            rname=u'test_role'
        )
        role.save()

        data = {
            'uname': u'netpolice',
            'addr': '127.0.0.0',
            'mask': '255.255.255.0',
            'rolid': role.pk
        }

        response = self.client.post(
            reverse('users.views.add_user'), data
        )

        self.assertRedirects(response, reverse('users.views.user_list'))

        user = User.objects.get(uname=data['uname'])

        self.assertEqual(type(user), User)


class EditUserViewTestCase(BaseViewTestCase):
    def test_edit_user_for_general(self):
        self.switch_current_admin_role(settings.ROLE_GENERAL)

        not_exist_user_pk = 9999

        response = self.client.get(
            reverse('users.views.edit_user', args=(not_exist_user_pk,))
        )
        self.assertEqual(response.status_code, httplib.FORBIDDEN)

    def test_edit_user_for_custom_not_found(self):
        self.switch_current_admin_role(settings.ROLE_CUSTOM)

        not_exist_user_pk = 9999

        response = self.client.get(
            reverse('users.views.edit_user', args=(not_exist_user_pk,))
        )
        self.assertEqual(response.status_code, httplib.NOT_FOUND)

    def test_edit_user_for_custom(self):
        self.switch_current_admin_role(settings.ROLE_CUSTOM)

        role = Role(
            rname=u'test_role'
        )
        role.save()

        new_user = User(
            uname=u'netpolice',
            addr='0',
            mask='0',
            rolid=role
        )
        new_user.save()

        response = self.client.get(
            reverse('users.views.edit_user', args=(new_user.pk,))
        )

        self.assertEqual(response.status_code, httplib.OK)
        self.assertTemplateUsed(response, 'users/edit_user.html')

        self.assertEqual(type(response.context['form']), UserForm)
        self.assertEqual(response.context['user'], new_user)

    def test_edit_user_send_data_success(self):
        self.switch_current_admin_role(settings.ROLE_CUSTOM)

        first_role = Role(
            rname=u'first_test_role'
        )
        first_role.save()

        second_role = Role(
            rname=u'second_test_role'
        )
        second_role.save()

        user = User(
            uname=u'netpolice',
            rolid=first_role
        )
        user.save()

        data = {
            'uname': u'netpolice',
            'addr': '127.0.0.0',
            'mask': '255.255.255.0',
            'rolid': second_role.pk
        }

        response = self.client.post(
            reverse(
                'users.views.edit_user', args=(user.pk,)
            ), data
        )

        self.assertRedirects(
            response,
            reverse('users.views.user_list')
        )

        user_from_db = User.objects.get(uname=data['uname'])

        self.assertEqual(user_from_db, user)
        self.assertEqual(user_from_db.uname, data['uname'])
        self.assertEqual(user_from_db.rolid, second_role)


class DeleteUserViewTestCase(BaseViewTestCase):
    def test_delete_user_for_general(self):
        self.switch_current_admin_role(settings.ROLE_GENERAL)

        not_exist_user_pk = 9999

        response = self.client.get(
            reverse('users.views.delete_user', args=(not_exist_user_pk,))
        )

        self.assertEqual(response.status_code, httplib.FORBIDDEN)

    def test_delete_user_for_custom_not_found(self):
        self.switch_current_admin_role(settings.ROLE_CUSTOM)

        not_exist_user_pk = 9999

        response = self.client.get(
            reverse('users.views.delete_user', args=(not_exist_user_pk,))
        )

        self.assertEqual(response.status_code, httplib.NOT_FOUND)

    def test_delete_user_for_custom_default_user(self):
        self.switch_current_admin_role(settings.ROLE_CUSTOM)

        role = Role(
            rname=u'test_role'
        )
        role.save()

        default_user = User(
            uname=u'',
            addr='0',
            mask='0',
            rolid=role
        )
        default_user.save()

        response = self.client.get(
            reverse('users.views.delete_user', args=(default_user.pk,))
        )

        self.assertEqual(response.status_code, httplib.FORBIDDEN)

    def test_delete_user_for_custom_success(self):
        self.switch_current_admin_role(settings.ROLE_CUSTOM)

        role = Role(
            rname=u'test_role'
        )
        role.save()

        new_user = User(
            uname=u'test_user',
            addr='0',
            mask='0',
            rolid=role
        )
        new_user.save()

        new_user_pk = new_user.pk

        self.assertEqual(new_user, User.objects.get(pk=new_user_pk))

        response = self.client.get(
            reverse('users.views.delete_user', args=(new_user_pk,))
        )

        self.assertRedirects(response, reverse('users.views.user_list'))

        self.assertRaises(
            User.DoesNotExist, User.objects.get, pk=new_user_pk
        )
