
#
# Copyright (c) 2011, MegaVersiya, LTD. All rights reserved.
#
# Redistribution and use in source and binary forms, with or
# without modification, are permitted provided that
# the following conditions are met:
# Redistributions of source code must retain the above copyright notice,
# this list of conditions and the following disclaimer.
# Redistributions in binary form must reproduce the above copyright notice,
# this list of conditions and the following disclaimer in the documentation
# and/or other materials provided with the distribution.
# Neither the name of the MegaVersiya, LTD nor the names of its contributors
# may be used to endorse or promote products derived from this software
# without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
# OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
# OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT * LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
# OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
# DAMAGE.
#

from django.shortcuts import render, get_object_or_404, redirect
from django.core.context_processors import csrf
from django.core.urlresolvers import reverse
from django.utils.translation import ugettext as _
from django.conf import settings
from django.contrib import messages

from core.http import Http403
from auth.decorators import login_required
from admins.models import Admin
from admins.forms import AddAdminForm, AdminPasswordChangeForm


@login_required
def admin_list(request):
    """Display administrator list."""
    current_admin = request.session['current_admin']

    is_general_admin = (current_admin.type == settings.ROLE_GENERAL)

    can_delete = can_add = is_general_admin

    admin_list = Admin.objects.get_list(
        is_general_admin, current_admin.pk
    )

    context = {
        'admin_list': admin_list,
        'current_admin': current_admin,
        'can_add': can_add,
        'can_delete': can_delete
    }

    return render(request, 'admins/admin_list.html', context)


@login_required
def add_admin(request):
    """Add new administrator."""
    current_admin = request.session['current_admin']

    if current_admin.type != settings.ROLE_GENERAL:
        raise Http403

    if request.method == 'POST':
        form = AddAdminForm(request.POST)

        if form.is_valid():
            form.save()

            messages.success(
                request, _(u'Admin was successfully added.')
            )

            return redirect(reverse('admins.views.admin_list'))
    else:
        form = AddAdminForm()

    context = {
        'form': form
    }
    context.update(csrf(request))

    return render(request, 'admins/add_admin.html', context)


@login_required
def change_password(request, admin_pk):
    """Change administrator password."""
    current_admin = request.session['current_admin']

    if current_admin.type != settings.ROLE_GENERAL:
        if int(admin_pk) != current_admin.pk:
            raise Http403

    admin = get_object_or_404(Admin, pk=admin_pk)

    if request.method == 'POST':
        form = AdminPasswordChangeForm(admin, request.POST)

        if form.is_valid():
            form.save()

            messages.success(
                request,
                _(u'Admin password was successfully changed.')
            )

            return redirect(reverse('admins.views.admin_list'))
    else:
        form = AdminPasswordChangeForm(admin)

    context = {
        'form': form
    }
    context.update(csrf(request))

    return render(request, 'admins/change_password.html', context)


@login_required
def delete_admin(request, admin_pk):
    """Delete administrator by pk."""
    current_admin = request.session['current_admin']

    if current_admin.type != settings.ROLE_GENERAL:
        raise Http403

    # Suicide is not accepted.
    if int(admin_pk) == current_admin.pk:
        raise Http403

    admin = get_object_or_404(Admin, pk=admin_pk)
    admin.delete()

    messages.success(request, _(u'Admin was successfully deleted.'))

    return redirect(reverse('admins.views.admin_list'))
