
#
# Copyright (c) 2011, MegaVersiya, LTD. All rights reserved.
#
# Redistribution and use in source and binary forms, with or
# without modification, are permitted provided that
# the following conditions are met:
# Redistributions of source code must retain the above copyright notice,
# this list of conditions and the following disclaimer.
# Redistributions in binary form must reproduce the above copyright notice,
# this list of conditions and the following disclaimer in the documentation
# and/or other materials provided with the distribution.
# Neither the name of the MegaVersiya, LTD nor the names of its contributors
# may be used to endorse or promote products derived from this software
# without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
# OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
# OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT * LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
# OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
# DAMAGE.
#

import logging
import urllib2

import suds

from utils.decorators import log_call, reraise

class LicenseException(Exception):
    """Base license utils exception."""
    def __init__(self, code=0, message=''):
        Exception.__init__(self, code, message)

        self._code = code
        self._message = unicode(message)

    @property
    def code(self):
        return self._code

    @property
    def message(self):
        return self._message


LICENSE_IS_VALID = 1

LICENSE_DEFAULT = 2

LICENSE_IS_NOT_VALID = 3


logger = logging.getLogger('license.utils')


class License(object):
    """License server api utils."""
    @reraise((suds.WebFault, urllib2.URLError), LicenseException)
    @log_call(logger)
    def __init__(self, wsdl_url, timeout):
        self.wsdl_url = wsdl_url
        self.timeout = timeout

        self.client = suds.client.Client(
            self.wsdl_url
        )
        self.client.set_options(timeout=timeout)

    @reraise((suds.WebFault, urllib2.URLError), LicenseException)
    @log_call(logger)
    def create_installation_id(
        self, owner_id, product_id, product_version):

        exec_result = self.client.service.CreateInstallationKey(
            owner_id, product_id, product_version
        )

        result = exec_result.CreateInstallationKeyResult

        if result.Code:
            raise LicenseException(
                result.Code,
                result.Message
            )

        return exec_result.installationKey.Id

    @reraise((suds.WebFault, urllib2.URLError), LicenseException)
    @log_call(logger)
    def activate_license(self, license_type, card_number, card_pin):
        #attrs = self.client.factory.create('ns3:AttributeCollection')
        attrs = self.client.factory.create('ns4:AttributeCollection')

        data = (('CardNumber', card_number), ('CardPin', card_pin))
        for attr_name, attr_value in data:
            attr = self.client.factory.create('ns4:Attribute')

            attr.Name = attr_name
            attr.Value = attr_value

            attrs.Attribute.append(attr)

        exec_result = self.client.service.Activate(
            type=license_type,
            attributes=attrs
        )

        result = exec_result.ActivateResult

        SUCCESS = 0
        ALREADY_ACTIVATED = 3

        if result.Code not in (SUCCESS, ALREADY_ACTIVATED):
            raise LicenseException(
                result.Code,
                result.Message
            )

        return exec_result.license.Id

    @reraise((suds.WebFault, urllib2.URLError), LicenseException)
    @log_call(logger)
    def link_installation_and_license(self, installation_id, license_id):
        exec_result = self.client.service.LinkInstallationAndLicense(
            installation_id, license_id
        )

        if exec_result.Code:
            raise LicenseException(
                exec_result.Code, exec_result.Message
            )

        return True
        
    @reraise((suds.WebFault, urllib2.URLError), LicenseException)
    @log_call(logger)
    def AssociateLicenseWithUser(self, OwnerId, license_type, card_number, card_pin):
        attrs = self.client.factory.create('ns4:AttributeCollection')
        data = (('CardNumber', card_number), ('CardPin', card_pin))
        for attr_name, attr_value in data:
            attr = self.client.factory.create('ns4:Attribute')

            attr.Name = attr_name
            attr.Value = attr_value

            attrs.Attribute.append(attr)

        exec_result = self.client.service.AssociateLicenseWithUser(
            userId=OwnerId, 
            type=license_type,
            attributes=attrs
        )
        result = exec_result.AssociateLicenseWithUserResult
        
        if result.Code != 0:
            raise LicenseException(
                result.Code, result.Message
            )
        
        return True

    @reraise((suds.WebFault, urllib2.URLError), LicenseException)
    @log_call(logger)
    def activate(self,
                  card_number,
                  card_pin,
                  license_type,
                  installation_id, OwnerId):
        license_id = self.activate_license(
            license_type, card_number, card_pin
        )

        self.link_installation_and_license(
            installation_id, license_id
        )
        
        self.AssociateLicenseWithUser(
            OwnerId,
            license_type, 
            card_number, 
            card_pin
        )

        return license_id

    @reraise((suds.WebFault, urllib2.URLError), LicenseException)
    @log_call(logger)
    def check_installation(self, installation_id):
        exec_result = self.client.service.CheckInstallationKey(
            installation_id
        )

        result = exec_result.CheckInstallationKeyResult
        if result.Code:
            raise LicenseException(
                result.Code,
                result.Message
            )

        return exec_result.isValid

    @reraise((suds.WebFault, urllib2.URLError), LicenseException)
    @log_call(logger)
    def check_license(self, license_id):
        exec_result = self.client.service.CheckLicense(license_id)

        result = exec_result.CheckLicenseResult

        if result.Code:
            raise LicenseException(
                result.Code,
                result.Message
            )

        return (exec_result.isValid, exec_result.expireDate)

    @reraise((suds.WebFault, urllib2.URLError), LicenseException)
    @log_call(logger)
    def get_license_info(self, license_type, card_number, card_pin):
        #attrs = self.client.factory.create('ns3:AttributeCollection')
        attrs = self.client.factory.create('ns4:AttributeCollection')

        data = (('CardNumber', card_number), ('CardPin', card_pin))
        for attr_name, attr_value in data:
            attr = self.client.factory.create('ns4:Attribute')

            attr.Name = attr_name
            attr.Value = attr_value

            attrs.Attribute.append(attr)

        exec_result = self.client.service.GetLicenseByActivationData(
            type=license_type,
            attributes=attrs
        )

        result = exec_result.GetLicenseByActivationDataResult
        if result.Code:
            raise LicenseException(
                result.Code,
                result.Message
            )

        license = exec_result.license

        data = {
            'Id': license.Id,
            'OwnerId': license.OwnerId,
            'StartDate': license.StartDate
        }

        for i in xrange(len(license.Attributes.Attribute)):
            attr = license.Attributes.Attribute[i]
            
            if hasattr(attr, 'Name'):
                name = attr.Name
                value = attr.Value if hasattr(attr, 'Value') else None
                data[name] = value

        return data
