
#
# Copyright (c) 2011, MegaVersiya, LTD. All rights reserved.
#
# Redistribution and use in source and binary forms, with or
# without modification, are permitted provided that
# the following conditions are met:
# Redistributions of source code must retain the above copyright notice,
# this list of conditions and the following disclaimer.
# Redistributions in binary form must reproduce the above copyright notice,
# this list of conditions and the following disclaimer in the documentation
# and/or other materials provided with the distribution.
# Neither the name of the MegaVersiya, LTD nor the names of its contributors
# may be used to endorse or promote products derived from this software
# without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
# OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
# OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT * LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
# OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
# DAMAGE.
#

from django.shortcuts import render, redirect
from django.core.urlresolvers import reverse
from django.core.context_processors import csrf
from django.conf import settings
from django.utils.translation import ugettext as _
from django.contrib import messages
from django.utils.encoding import force_unicode

from core.http import Http403
from auth.decorators import login_required
from license.lib.licenseutils import License, LicenseException
from license.lib.licenseutils import LICENSE_IS_VALID, \
                               LICENSE_DEFAULT, \
                               LICENSE_IS_NOT_VALID
from license.forms import CreateInstallationForm, ActivateLicenseForm


@login_required
def main(request):
    """Display license information."""

    installation_id = settings.LICENSE_INSTALLATION_ID.get()
    license_id = settings.LICENSE_ID.get()

    installation_exists = bool(installation_id)
    license_exists = bool(license_id)

    context = {
        'installation_exists': installation_exists,
        'license_exists': license_exists,
        'installation_id': installation_id,
        'license_id': license_id
    }

    if installation_exists:
        try:
            service = License(
                settings.LICENSE_SERVER_WSDL_URL,
                settings.LICENSE_SERVER_TIMEOUT
            )

            checking_result = service.check_installation(installation_id)

            code_to_message = {
                LICENSE_IS_VALID: _(u'License is valid.'),
                LICENSE_DEFAULT: _(u'Default license.'),
                LICENSE_IS_NOT_VALID: _(u'License is not valid.')
            }

            context['message'] = code_to_message[checking_result]

            if license_exists:
                if checking_result == LICENSE_IS_NOT_VALID:
                    is_valid, expire_date = service.check_license(
                        license_id
                    )

                    context['is_valid'] = is_valid
                    context['expire_date'] = expire_date

                license_type = settings.LICENSE_TYPE
                card_number = settings.LICENSE_CARD_NUMBER.get()
                card_pin = settings.LICENSE_CARD_PIN.get()

                license_info = service.get_license_info(
                    license_type, card_number, card_pin
                )
                context['license_info'] = license_info
        except LicenseException, exc:
            return render(request, 'license/error.html', {
                'exception': exc
            })
    else:
        context['message'] = _('Installation does not exists.')

    return render(request, 'license/main.html', context)


@login_required
def create_installation(request):
    """Create product installation."""
    installation_id = settings.LICENSE_INSTALLATION_ID.get()
    if installation_id:
        raise Http403

    if request.method == 'POST':
        form = CreateInstallationForm(request.POST)          

        if form.is_valid():
            owner_id = form.cleaned_data['owner_id']
            product_id = form.cleaned_data['product_id']
            product_version = form.cleaned_data['product_version']

            try:
                service = License(
                    settings.LICENSE_SERVER_WSDL_URL,
                    settings.LICENSE_SERVER_TIMEOUT
                )

                installation_id = service.create_installation_id(
                    owner_id, product_id, product_version
                )

                settings.LICENSE_INSTALLATION_ID.set(installation_id)
                settings.LICENSE_OWNER_ID.set(owner_id)
                settings.LICENSE_PRODUCT_ID.set(product_id)
                settings.LICENSE_PRODUCT_VERSION.set(
                    force_unicode(product_version)
                )

                messages.success(
                    request, _(u'Installation was successfully created.')
                )

                return redirect(reverse('license.views.main'))

            except LicenseException, exc:
                return render(request, 'license/error.html', {
                    'exception': exc
                })
    else:
        
        form = CreateInstallationForm(
            initial={
                'product_id': settings.LICENSE_PRODUCT_ID.get(), 
                'product_version': settings.LICENSE_PRODUCT_VERSION.get()
            }
        )

    context = {
        'form': form
    }
    context.update(csrf(request))

    return render(request, 'license/installation_form.html', context)


@login_required
def activate_license(request):
    """Activate product license."""
    license_id = settings.LICENSE_ID.get()
    if license_id:
        raise Http403

    if request.method == 'POST':
        form = ActivateLicenseForm(request.POST)

        if form.is_valid():
            card_number = form.cleaned_data['card_number']
            card_pin = form.cleaned_data['card_pin']

            license_type = settings.LICENSE_TYPE
            installation_id = settings.LICENSE_INSTALLATION_ID.get()

            try:
                service = License(
                    settings.LICENSE_SERVER_WSDL_URL,
                    settings.LICENSE_SERVER_TIMEOUT
                )

                license_id = service.activate(
                    card_number,
                    card_pin,
                    license_type,
                    installation_id, 
                    settings.LICENSE_OWNER_ID.get()
                )

                settings.LICENSE_CARD_NUMBER.set(
                    force_unicode(card_number)
                )
                settings.LICENSE_CARD_PIN.set(card_pin)
                settings.LICENSE_ID.set(license_id)

                messages.success(
                    request, _(u'License was successfully activated.')
                )

                return redirect(reverse('license.views.main'))

            except LicenseException, exc:
                return render(request, 'license/error.html', {
                    'exception': exc
                })
    else:
        form = ActivateLicenseForm()

    context = {
        'form': form
    }
    context.update(csrf(request))

    return render(request, 'license/activation_form.html', context)
