
#
# Copyright (c) 2011, MegaVersiya, LTD. All rights reserved.
#
# Redistribution and use in source and binary forms, with or
# without modification, are permitted provided that
# the following conditions are met:
# Redistributions of source code must retain the above copyright notice,
# this list of conditions and the following disclaimer.
# Redistributions in binary form must reproduce the above copyright notice,
# this list of conditions and the following disclaimer in the documentation
# and/or other materials provided with the distribution.
# Neither the name of the MegaVersiya, LTD nor the names of its contributors
# may be used to endorse or promote products derived from this software
# without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
# OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
# OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT * LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
# OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
# DAMAGE.
#

import httplib

from django.core.urlresolvers import reverse
from django.test import TestCase
from django.conf import settings
from django.utils.translation import ugettext_lazy as _

from utils.test import BaseViewTestCase
from roles.models import Category
from manual_categorization.models import Domain, DomainCategory
from manual_categorization.forms import DomainForm


class NoAuthManualCategorizationViewTestCase(TestCase):
    def test_domain_list(self):
        response = self.client.get(
            reverse('manual_categorization.views.domain_list')
        )

        self.assertRedirects(response, settings.LOGIN_URL)

    def test_add_domain(self):
        response = self.client.get(
            reverse('manual_categorization.views.add_domain')
        )

        self.assertRedirects(response, settings.LOGIN_URL)

    def test_edit_domain(self):
        response = self.client.get(
            reverse('manual_categorization.views.edit_domain', args=(1,))
        )

        self.assertRedirects(response, settings.LOGIN_URL)

    def test_delete_domain(self):
        response = self.client.get(
            reverse('manual_categorization.views.delete_domain', args=(1,))
        )

        self.assertRedirects(response, settings.LOGIN_URL)


class DomainListViewTestCase(BaseViewTestCase):
    def test_domain_list(self):
        response = self.client.get(
            reverse('manual_categorization.views.domain_list')
        )

        self.assertEqual(response.status_code, httplib.OK)
        self.assertTemplateUsed(
            response, 'manual_categorization/domain_list.html'
        )

        self.assertTrue('domain_list' in response.context)


class AddDomainViewTestCase(BaseViewTestCase):
    def test_add_domain(self):
        response = self.client.get(
            reverse('manual_categorization.views.add_domain')
        )

        self.assertEqual(response.status_code, httplib.OK)

        self.assertTemplateUsed(
            response, 'manual_categorization/add_domain.html'
        )
        self.assertTemplateUsed(
            response, 'manual_categorization/form.html'
        )

        self.assertTrue('domain_form' in response.context)
        self.assertEqual(
            type(response.context['domain_form']), DomainForm
        )

    def test_add_domain_send_empty_data(self):
        data = {
            'dname': u''
        }

        response = self.client.post(
            reverse('manual_categorization.views.add_domain'), data
        )

        self.assertEqual(response.status_code, httplib.OK)

        self.assertTemplateUsed(
            response, 'manual_categorization/add_domain.html'
        )
        self.assertTemplateUsed(
            response, 'manual_categorization/form.html'
        )

        self.assertTrue('domain_form' in response.context)
        self.assertEqual(
            type(response.context['domain_form']), DomainForm
        )

        self.assertFormError(
            response, 'domain_form', 'dname',
            _(u'This field is required.')
        )

    def test_add_category_send_success(self):
        category = Category(
            cat=1000,
            cname=u'Test category'
        )
        category.save()

        data = {
            'dname': u'example.com',
            'category_%d' % category.pk: True
        }

        response = self.client.post(
            reverse('manual_categorization.views.add_domain'), data
        )

        self.assertRedirects(
            response,
            reverse('manual_categorization.views.domain_list')
        )

        domain = Domain.objects.get(dname=data['dname'])

        self.assertEqual(domain.dname, data['dname'])

        domain_category = DomainCategory.objects.get(
            domain=domain, cat=category
        )
        self.assertTrue(domain_category)


class EditDomainViewTestCase(BaseViewTestCase):
    def test_edit_domain_not_found(self):
        response = self.client.get(
            reverse('manual_categorization.views.edit_domain', args=(1,))
        )

        self.assertEqual(response.status_code, httplib.NOT_FOUND)

    def test_edit_domain(self):
        domain = Domain(
            dname=u'example.com'
        )
        domain.save()

        response = self.client.get(
            reverse(
                'manual_categorization.views.edit_domain',
                args=(domain.pk,)
            )
        )

        self.assertEqual(response.status_code, httplib.OK)

        self.assertTemplateUsed(
            response, 'manual_categorization/edit_domain.html'
        )
        self.assertTemplateUsed(
            response, 'manual_categorization/form.html'
        )

        self.assertTrue('domain_form' in response.context)
        self.assertEqual(
            type(response.context['domain_form']), DomainForm
        )

    def test_edit_domain_send_empty_data(self):
        domain = Domain(
            dname=u'example.com'
        )
        domain.save()

        data = {
            'dname': u''
        }

        response = self.client.post(
            reverse(
                'manual_categorization.views.edit_domain',
                args=(domain.pk,)
            ), data
        )

        self.assertEqual(response.status_code, httplib.OK)

        self.assertTemplateUsed(
            response, 'manual_categorization/edit_domain.html'
        )
        self.assertTemplateUsed(
            response, 'manual_categorization/form.html'
        )

        self.assertTrue('domain_form' in response.context)
        self.assertEqual(
            type(response.context['domain_form']), DomainForm
        )

        self.assertFormError(
            response, 'domain_form', 'dname',
            _(u'This field is required.')
        )

    def test_edit_domain_send_success(self):
        domain = Domain(
            dname=u'example.com'
        )
        domain.save()

        category = Category(
            cat=1000,
            cname=u'Test category'
        )
        category.save()

        data = {
            'dname': u'example2.com',
            'category_%d' % category.pk: True
        }

        response = self.client.post(
            reverse(
                'manual_categorization.views.edit_domain',
                args=(domain.pk,)
            ), data
        )

        self.assertRedirects(
            response,
            reverse('manual_categorization.views.domain_list')
        )

        domain = Domain.objects.get(dname=data['dname'])

        self.assertEqual(domain.dname, data['dname'])

        domain_category = DomainCategory.objects.get(
            domain=domain, cat=category
        )
        self.assertTrue(domain_category)


class DeleteDomainViewTestCase(BaseViewTestCase):
    def test_delete_domain_not_found(self):
        response = self.client.get(
            reverse(
                'manual_categorization.views.delete_domain',
                args=(1,)
            )
        )

        self.assertEqual(response.status_code, httplib.NOT_FOUND)

    def test_delete_domain_success(self):
        domain = Domain(
            dname=u'example.com'
        )
        domain.save()

        domain_pk = domain.pk

        self.assertEqual(domain, Domain.objects.get(pk=domain_pk))

        response = self.client.get(
            reverse(
                'manual_categorization.views.delete_domain',
                args=(domain_pk,)
            )
        )

        self.assertRedirects(
            response,
            reverse('manual_categorization.views.domain_list')
        )

        self.assertRaises(
            Domain.DoesNotExist, Domain.objects.get, pk=domain_pk
        )
