
#
# Copyright (c) 2011, MegaVersiya, LTD. All rights reserved.
#
# Redistribution and use in source and binary forms, with or
# without modification, are permitted provided that
# the following conditions are met:
# Redistributions of source code must retain the above copyright notice,
# this list of conditions and the following disclaimer.
# Redistributions in binary form must reproduce the above copyright notice,
# this list of conditions and the following disclaimer in the documentation
# and/or other materials provided with the distribution.
# Neither the name of the MegaVersiya, LTD nor the names of its contributors
# may be used to endorse or promote products derived from this software
# without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
# OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
# OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT * LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
# OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
# DAMAGE.
#

import httplib

from django.core.urlresolvers import reverse
from django.test import TestCase
from django.conf import settings
from django.utils.translation import ugettext as _

from utils.test import BaseViewTestCase
from roles.models import Role, Hierarchy
from roles.forms import RoleForm, ParentRolesForm, CategoryListForm


class NoAuthAdminViewTestCase(TestCase):
    def test_role_list(self):
        response = self.client.get(reverse('roles.views.role_list'))

        self.assertRedirects(response, settings.LOGIN_URL)

    def test_add_role(self):
        response = self.client.get(reverse('roles.views.add_role'))

        self.assertRedirects(response, settings.LOGIN_URL)

    def test_view_role(self):
        response = self.client.get(
            reverse('roles.views.view_role', args=(1,))
        )

        self.assertRedirects(response, settings.LOGIN_URL)

    def test_edit_role(self):
        response = self.client.get(
            reverse('roles.views.edit_role', args=(1,))
        )

        self.assertRedirects(response, settings.LOGIN_URL)

    def test_delete_role(self):
        response = self.client.get(
            reverse('roles.views.delete_role', args=(1,))
        )

        self.assertRedirects(response, settings.LOGIN_URL)

    def test_url_list(self):
        response = self.client.get(
            reverse('roles.views.url_list', args=(1, 'accepted'))
        )

        self.assertRedirects(response, settings.LOGIN_URL)

    def test_delete_list_item(self):
        response = self.client.get(
            reverse(
                'roles.views.delete_url_list_item',
                args=(1, 1)
            )
        )

        self.assertRedirects(response, settings.LOGIN_URL)

    def test_categories(self):
        response = self.client.get(
            reverse(
                'roles.views.categories',
                args=(1, 'accepted')
            )
        )

        self.assertRedirects(response, settings.LOGIN_URL)

    def test_parent_roles(self):
        response = self.client.get(
            reverse(
                'roles.views.parent_roles',
                args=(1,)
            )
        )

        self.assertRedirects(response, settings.LOGIN_URL)


class RoleListsViewTestCase(BaseViewTestCase):
    def test_empty_role_list(self):
        response = self.client.get(reverse('roles.views.role_list'))

        self.assertEqual(response.status_code, httplib.OK)
        self.assertTemplateUsed(response, 'roles/role_list.html')

        self.assertTrue('role_list' in response.context)
        self.assertEqual(len(response.context['role_list']), 0)


class AddRoleViewTestCase(BaseViewTestCase):
    def test_add_role(self):
        response = self.client.get(reverse('roles.views.add_role'))

        self.assertEqual(response.status_code, httplib.OK)

        self.assertTemplateUsed(response, 'roles/add_role.html')

        self.assertTrue('role_form' in response.context)
        self.assertEqual(type(response.context['role_form']), RoleForm)

        self.assertTrue('parent_roles_form' in response.context)
        self.assertEqual(
            type(response.context['parent_roles_form']), ParentRolesForm
        )

    def test_add_role_send_empty_data(self):
        data = {
            'rname': '',
            'type': settings.ROLE_CUSTOM
        }

        response = self.client.post(
            reverse('roles.views.add_role'), data
        )

        self.assertFormError(
            response, 'role_form', 'rname',
            _(u'This field is required.')
        )

    def test_add_role_send_success(self):
        parent_role = Role(
            rname='parent_role',
            type=settings.ROLE_GENERAL
        )
        parent_role.save()

        data = {
            'rname': 'role_name',
            'type': settings.ROLE_CUSTOM,
            'role_%d' % parent_role.pk: 'True'
        }

        response = self.client.post(
            reverse('roles.views.add_role'), data
        )

        self.assertRedirects(
            response, reverse('roles.views.role_list')
        )

        role = Role.objects.get(rname=data['rname'])

        self.assertEqual(role.rname, data['rname'])
        self.assertEqual(role.type, data['type'])

        hierarchy_item = Hierarchy.objects.get(
            parent=parent_role.pk, child=role.pk
        )
        self.assertTrue(hierarchy_item)


class EditRoleTestCase(BaseViewTestCase):
    def test_edit_role_not_found(self):
        response = self.client.get(
            reverse('roles.views.edit_role', args=(1,))
        )

        self.assertEqual(response.status_code, httplib.NOT_FOUND)

    def test_edit_role(self):
        role = Role(
            rname=u'Role',
            type=settings.ROLE_CUSTOM
        )
        role.save()

        response = self.client.get(
            reverse('roles.views.edit_role', args=(role.pk,))
        )

        self.assertEqual(response.status_code, httplib.OK)

        self.assertTemplateUsed(response, 'roles/edit_role.html')

        self.assertTrue('form' in response.context)
        self.assertEqual(type(response.context['form']), RoleForm)


class ViewRoleViewTestCase(BaseViewTestCase):
    def test_view_role(self):
        role = Role(
            rname=u'Role',
            type=settings.ROLE_CUSTOM
        )
        role.save()

        response = self.client.get(
            reverse('roles.views.view_role', args=(role.pk,))
        )

        self.assertEqual(response.status_code, httplib.OK)

        self.assertTemplateUsed(response, 'roles/view_role.html')

        self.assertTrue('role' in response.context)
        self.assertEquals(response.context['role'], role)

        self.assertTrue('white_list' in response.context)
        self.assertTrue('black_list' in response.context)
        self.assertTrue('accepted_cat_list' in response.context)
        self.assertTrue('rejected_cat_list' in response.context)
        self.assertTrue('redirected_cat_list' in response.context)


class ParentRolesViewTestCase(BaseViewTestCase):
    def test_parent_roles_not_found(self):
        response = self.client.get(
            reverse('roles.views.parent_roles', args=(1,))
        )

        self.assertEqual(response.status_code, httplib.NOT_FOUND)

    def test_parent_roles(self):
        role = Role(
            rname=u'Role',
            type=settings.ROLE_CUSTOM
        )
        role.save()

        response = self.client.get(
            reverse('roles.views.parent_roles', args=(role.pk,))
        )

        self.assertEqual(response.status_code, httplib.OK)

        self.assertTemplateUsed(response, 'roles/parent_roles.html')

        self.assertTrue('role' in response.context)
        self.assertEquals(response.context['role'], role)

        self.assertTrue('form' in response.context)
        self.assertEquals(type(response.context['form']), ParentRolesForm)


class CategoriesViewTestCase(BaseViewTestCase):
    def test_categories_role_not_found(self):
        response = self.client.get(
            reverse('roles.views.categories', args=(1, 'rejected'))
        )

        self.assertEqual(response.status_code, httplib.NOT_FOUND)

    def test_categories_unknown_action_type(self):
        role = Role(
            rname=u'Role',
            type=settings.ROLE_CUSTOM
        )
        role.save()

        response = self.client.get(
            reverse('roles.views.categories', args=(role.pk, 'unknown'))
        )

        self.assertEqual(response.status_code, httplib.NOT_FOUND)

    def test_categories(self):
        role = Role(
            rname=u'Role',
            type=settings.ROLE_CUSTOM
        )
        role.save()

        response = self.client.get(
            reverse('roles.views.categories', args=(role.pk, 'rejected'))
        )

        self.assertEqual(response.status_code, httplib.OK)

        self.assertTemplateUsed(response, 'roles/categories.html')

        self.assertTrue('role' in response.context)
        self.assertEquals(response.context['role'], role)

        self.assertTrue('form' in response.context)
        self.assertEqual(type(response.context['form']), CategoryListForm)


class DeleteRoleViewTestCase(BaseViewTestCase):
    def test_delete_role_not_found(self):
        not_exist_role_pk = 9999

        response = self.client.get(
            reverse('roles.views.delete_role', args=(not_exist_role_pk,))
        )

        self.assertEqual(response.status_code, httplib.NOT_FOUND)

    def test_delete_role_not_the_same_type_forbidden(self):
        self.switch_current_admin_role(settings.ROLE_GENERAL)

        role = Role(
            rname='test_role',
            type=settings.ROLE_CUSTOM
        )
        role.save()

        response = self.client.get(
            reverse('roles.views.delete_role', args=(role.pk,))
        )

        self.assertEqual(response.status_code, httplib.FORBIDDEN)

    def test_delete_role_success(self):
        self.switch_current_admin_role(settings.ROLE_CUSTOM)

        role = Role(
            rname='test_role',
            type=settings.ROLE_CUSTOM
        )
        role.save()

        role_pk = role.pk

        self.assertEqual(role, Role.objects.get(pk=role_pk))

        response = self.client.get(
            reverse('roles.views.delete_role', args=(role.pk,))
        )

        self.assertRedirects(
            response, reverse('roles.views.role_list')
        )

        self.assertRaises(
            Role.DoesNotExist, Role.objects.get, pk=role_pk
        )
