
#
# Copyright (c) 2011, MegaVersiya, LTD. All rights reserved.
#
# Redistribution and use in source and binary forms, with or
# without modification, are permitted provided that
# the following conditions are met:
# Redistributions of source code must retain the above copyright notice,
# this list of conditions and the following disclaimer.
# Redistributions in binary form must reproduce the above copyright notice,
# this list of conditions and the following disclaimer in the documentation
# and/or other materials provided with the distribution.
# Neither the name of the MegaVersiya, LTD nor the names of its contributors
# may be used to endorse or promote products derived from this software
# without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
# OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
# OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT * LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
# OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
# DAMAGE.
#

import datetime

from django import forms
from django.db.models import Q
from django.utils.translation import ugettext_lazy as _
from django.conf import settings
from django.forms.extras.widgets import SelectDateWidget

from users.models import User
from roles.models import Role, Category


def get_user_queryset():
    return User.objects.exclude(
        Q(uname='') & ~Q(addr=0) & ~Q(mask=0)
    )

TABLE_FORMAT = 't'

CHART_FORMAT = 'c'


ITEMS_PER_PAGE_CHOICES = (
    (50, 50),
    (100, 100),
    (250, 250),
    (500, 500),
    (1000, 1000)
)


def get_action_list():
    action_list = list(settings.ACTION_TYPES)
    action_list.insert(0, ('', _(u'All')))

    return action_list


def get_year_range():
    current_year = datetime.date.today().year

    return range(current_year - 5, current_year + 1)


def get_from_date(days=7):
    return (datetime.date.today() - datetime.timedelta(days=days))


def get_to_date():
    return datetime.date.today()


class FilterForm(forms.Form):
    """Form for filtering statistics data."""

    user = forms.ModelChoiceField(
        queryset=get_user_queryset(),
        empty_label=_('All'),
        required=False,
        label=_(u'User'),
    )

    role = forms.ModelChoiceField(
        queryset=Role.objects.all(),
        empty_label=_('All'),
        required=False,
        label=_(u'Role'),
    )

    category = forms.ModelChoiceField(
        queryset=Category.objects.all(),
        empty_label=_(u'All'),
        required=False,
        label=_(u'Category'),
    )

    action = forms.ChoiceField(
        required=False,
        label=_(u'Action type'),
        choices=get_action_list()
    )

    from_date = forms.DateField(
        required=False,
        widget=SelectDateWidget(
            years=get_year_range()
        ),
        label=_(u'From date'),
        initial=lambda: get_from_date()
    )

    to_date = forms.DateField(
        required=False,
        widget=SelectDateWidget(
            years=get_year_range(),
        ),
        label=_(u'To date'),
        initial=lambda: get_to_date()
    )

    url = forms.CharField(
        required=False,
        label=_(u'URL'),
        help_text=_('Can be set as a regular expression')
    )

    client_ip = forms.CharField(
        required=False,
        label=_(u'IP address'),
        help_text=_('Can be set as a regular expression')
    )

    format = forms.CharField(
        required=True,
        label=_(u'Display format'),
        initial='t',
        widget=forms.RadioSelect(choices=(
            (TABLE_FORMAT, _(u'Table')),
            (CHART_FORMAT, _(u'Chart'))
        ))
    )
    items_per_page = forms.ChoiceField(
        required=True,
        label=_(u'Items per page'),
        choices=ITEMS_PER_PAGE_CHOICES
    )

    def clean(self):
        format = self.cleaned_data['format']

        from_date = self.cleaned_data.get('from_date')
        to_date = self.cleaned_data.get('to_date')

        if format == CHART_FORMAT:
            if not from_date or not to_date:
                raise forms.ValidationError(
                    _(u'You should select the correct date range.')
                )

        if from_date and to_date:
            delta = (to_date - from_date)
            if delta.days <= 0:
                raise forms.ValidationError(
                    _(u'You should select the correct date range.')
                )

        return self.cleaned_data
