
#
# Copyright (c) 2011, MegaVersiya, LTD. All rights reserved.
#
# Redistribution and use in source and binary forms, with or
# without modification, are permitted provided that
# the following conditions are met:
# Redistributions of source code must retain the above copyright notice,
# this list of conditions and the following disclaimer.
# Redistributions in binary form must reproduce the above copyright notice,
# this list of conditions and the following disclaimer in the documentation
# and/or other materials provided with the distribution.
# Neither the name of the MegaVersiya, LTD nor the names of its contributors
# may be used to endorse or promote products derived from this software
# without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
# OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
# OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT * LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
# OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
# DAMAGE.
#

from django import forms
from django.utils.translation import ugettext as _
from ipaddr import IPAddress, IPNetwork

from users.models import User


class UserForm(forms.ModelForm):
    """Form for create/edit user."""
    uname = forms.ChoiceField(
        required=False,
        label=_(u'User name')
    )

    addr = forms.IPAddressField(
        required=False,
        label=_(u'Address')
    )

    mask = forms.IPAddressField(
        required=False,
        label=_(u'Mask')
    )

    class Meta:
        model = User
        exclude = ('useid',)
        unique_together = ('uname', 'addr', 'mask')

    def __init__(self, user_login_list, *args, **kwargs):
        instance = kwargs.get('instance')

        if instance:
            if instance.addr:
                instance.addr = str(IPAddress(instance.addr))
            elif instance.addr == 0:
                instance.addr = ''

            if instance.mask:
                instance.mask = str(IPAddress(instance.mask))
            elif instance.mask == 0:
                instance.mask = ''

        super(UserForm, self).__init__(*args, **kwargs)

        self.fields['uname'].choices = user_login_list

        self.fields['rolid'].label = _(u'Role')

        self.is_default = User.objects.is_default_user(instance)

        if self.is_default:
            # Disable some fields when edit default user.
            fields_to_disable = ('uname', 'addr', 'mask')
            for field in fields_to_disable:
                self.fields[field].required = False
                self.fields[field].widget.attrs['readonly'] = True

    def clean_mask(self):
        mask = self.cleaned_data['mask']
        if mask:
            return int(IPAddress(mask))

        return 0

    def clean_addr(self):
        addr = self.cleaned_data['addr']
        mask = self.data.get('mask')

        if addr and mask:
            try:
                return int(IPNetwork('%s/%s' % (addr, mask)).network)
            except ValueError:
                raise forms.ValidationError(
                    _(u'Wrong network address or mask.')
                )

        return 0

    def clean(self):
        uname = self.cleaned_data.get('uname')
        addr = self.cleaned_data.get('addr')
        mask = self.cleaned_data.get('mask')

        instance = getattr(self, 'instance', None)

        # Check unique for uname + addr + mask.
        same_user = User.objects.get_by_params(uname, addr, mask)

        # New user.
        if not instance.pk:
            condition = same_user
        else:
            condition = same_user and (instance.pk != same_user.pk)

        if condition:
            raise forms.ValidationError(_(u'User already exists.'))

        # We should skip uname/addr validation for default user.
        if self.is_default or (uname or bool(addr and mask)):
            return self.cleaned_data

        raise forms.ValidationError(
            _(u'User name or address and mask pair must be given.')
        )
