
#
# Copyright (c) 2011, MegaVersiya, LTD. All rights reserved.
#
# Redistribution and use in source and binary forms, with or
# without modification, are permitted provided that
# the following conditions are met:
# Redistributions of source code must retain the above copyright notice,
# this list of conditions and the following disclaimer.
# Redistributions in binary form must reproduce the above copyright notice,
# this list of conditions and the following disclaimer in the documentation
# and/or other materials provided with the distribution.
# Neither the name of the MegaVersiya, LTD nor the names of its contributors
# may be used to endorse or promote products derived from this software
# without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
# OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
# OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT * LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
# OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
# DAMAGE.
#

from django.shortcuts import render, get_object_or_404, redirect
from django.core.context_processors import csrf
from django.core.urlresolvers import reverse
from django.core.cache import cache
from django.utils.translation import ugettext as _
from django.conf import settings
from django.contrib import messages

from core.http import Http403
from auth.decorators import login_required
from users.lib.userstorage import storage
from users.models import User
from users.forms import UserForm


@login_required
def user_list(request):
    """All users list."""
    current_admin = request.session['current_admin']

    user_list = User.objects.get_all_join_roles()

    user_limit_exhausted = (
        settings.LIMITS_USER_COUNT > 0 and
        len(user_list) >= settings.LIMITS_USER_COUNT
    )

    is_custom_admin = (current_admin.type == settings.ROLE_CUSTOM)

    can_delete = can_edit = is_custom_admin
    can_add = (is_custom_admin and not user_limit_exhausted)

    if user_limit_exhausted:
        messages.warning(
            request, _(u'User count limit has been exhausted.')
        )

    context = {
        'user_list': user_list,
        'user_login_list': [u['login'] for u in storage.all()],
        'can_delete': can_delete,
        'can_add': can_add,
        'can_edit': can_edit
    }

    return render(request, 'users/user_list.html', context)


@login_required
def add_user(request):
    """Add new user."""
    current_admin = request.session['current_admin']

    if current_admin.type != settings.ROLE_CUSTOM:
        raise Http403

    user_limit_exhausted = (
        settings.LIMITS_USER_COUNT > 0 and
        User.objects.count() >= settings.LIMITS_USER_COUNT
    )

    if user_limit_exhausted:
        raise Http403

    user_login_list = [(u['login'], u['login']) for u in storage.all()]
    user_login_list.insert(0, ('', '-' * 10))

    if request.method == 'POST':
        form = UserForm(user_login_list, request.POST)
        if form.is_valid():
            form.save()

            messages.success(request, _(u'User was successfully added.'))

            return redirect(reverse('users.views.user_list'))
    else:
        form = UserForm(user_login_list)

    context = {
        'form': form
    }
    context.update(csrf(request))

    return render(request, 'users/add_user.html', context)


@login_required
def edit_user(request, user_pk):
    """Edit one user by pk."""
    current_admin = request.session['current_admin']

    if current_admin.type != settings.ROLE_CUSTOM:
        raise Http403

    user = get_object_or_404(User, pk=user_pk)

    user_login_list = [(u['login'], u['login']) for u in storage.all()]
    user_login_list.insert(0, ('', '-' * 10))

    if request.method == 'POST':
        form = UserForm(user_login_list, request.POST, instance=user)

        if form.is_valid():
            form.save()

            cache.clear()

            messages.success(request, _(u'User was successfully edited.'))

            return redirect(reverse('users.views.user_list'))
    else:
        form = UserForm(user_login_list, instance=user)

    context = {
        'user': user,
        'form': form
    }
    context.update(csrf(request))

    return render(request, 'users/edit_user.html', context)


@login_required
def delete_user(request, user_pk):
    """Delete one user by pk."""
    current_admin = request.session['current_admin']

    if current_admin.type != settings.ROLE_CUSTOM:
        raise Http403

    user = get_object_or_404(User, pk=user_pk)

    # Default user can't be deleted.
    if user.is_default():
        raise Http403

    user.delete()

    cache.clear()

    messages.success(request, _(u'User was successfully deleted.'))

    return redirect(reverse('users.views.user_list'))
